'''macros.py: Generate macro values from configuration values and provide
substitution functions.

The following macros are available:

  LCODE CCODE PCCODE PKGNAME PKGCODE LANG COUNTRY PCOUNTRY RELEASE UPLOADER
  DATE TIMESTAMP SUPDEPS CLASS CLASSNAME CLASSNAMESPACE CLASSDEPENDS
'''

import time, re, os.path, os
from decimal import Decimal

def _file_map(file, key, sep = None):
    '''Look up key in given file ("key value" lines). Throw an exception if
    key was not found.'''

    val = None
    for l in open(file):
        try:
            (k, v) = l.split(sep)
        except ValueError:
            continue
        # sort out comments
        if k.find('#') >= 0 or v.find('#') >= 0:
            continue
        if k == key:
            val = v.strip()
    if val == None:
        raise KeyError, 'Key %s not found in %s' % (key, file)
    return val

class LangpackMacros:
    def __init__(self, locale, cls, release, version):
        '''Initialize values of macros.

        This uses information from maps/, config/, some hardcoded aggregate
        strings (such as package names), and some external input:
        
        - locale: Standard locale representation (e. g. pt_BR.UTF-8)
        - cls: empty, or valid name in maps/classnames
        - release: distrorelease name (in maps/releaseversions)
        - version: export date of language pack tarball, will be used in
          version numbers
        '''
        self.macros = {}
        # chop .* and @* suffixes to get encoding-agnostic locale
        try:
            self['LOCALE'] = (locale.split('.')[0]).split('@')[0]
        except KeyError:
            self['LOCALE'] = locale

        # class
        self['CLASS'] = cls
        if cls:
            self['CLASSNAME'] = _file_map('maps/classnames', cls)
            self['CLASSNAMESPACE'] = self['CLASSNAME'] + ' '

        # language and country
        try:
            (self['LCODE'], self['CCODE']) = self['LOCALE'].split('_')
            self['PCCODE'] = '(%s)' % self['CCODE']
            try:
                self['COUNTRY'] = _file_map('maps/countries', self['CCODE'], ':')
                self['PCOUNTRY'] = '(%s)' % self['COUNTRY']
            except KeyError:
                self['COUNTRY'] = ''
                self['PCOUNTRY'] = ''
        except ValueError:
            self['LCODE'] = self['LOCALE']

        if not self['LCODE']:
            raise Exception, 'Internal error: LCODE is empty'

        # other variables
        self['DATE'] = time.strftime('%a, %d %b %Y %H:%M:%S +0000')
        self['RELEASE'] = release
        self['RELEASEVERSION'] = _file_map('maps/releaseversions', release.split('-')[0])
        self['TIMESTAMP'] = version


        # package name
        if Decimal(self['RELEASEVERSION']) >= Decimal('9.10'):
            try:
                self['PKGCODE'] = _file_map('maps/locale2pkgname', '%s-%s' % (self['LCODE'], self['CCODE'].lower()), ':')
                self['LANG'] = _file_map('maps/variants', '%s' % self['PKGCODE'], ':')
                if self['CLASS']:
                    self['PKGNAME'] = '%s-%s' % (self['CLASS'], self['PKGCODE'])
                else:
                    self['PKGNAME'] = self['PKGCODE']
            except KeyError:
                self['PKGCODE'] = self['LCODE']
                try:
                    self['LANG'] = _file_map('maps/variants', '%s' % self['PKGCODE'], ':')
                except KeyError:
                    self['LANG'] = _file_map('maps/languages', self['LCODE'], ':')
                if self['CLASS']:
                    self['PKGNAME'] = '%s-%s' % (self['CLASS'], self['LCODE'])
                else:
                    self['PKGNAME'] = self['LCODE']
        else:
            self['PKGCODE'] = self['LCODE']
            self['LANG'] = _file_map('maps/languages', self['LCODE'], ':')
            if self['CLASS']:
                self['PKGNAME'] = '%s-%s' % (self['CLASS'], self['LCODE'])
            else:
                self['PKGNAME'] = self['LCODE']

        # %CLASS%-base packages should depend on -base       
        if cls != '':
            self['CLASSDEPENDS'] = ', language-pack-%s-base' % self['PKGCODE']
        else:
            self['CLASSDEPENDS'] = ''

        # configuration files
        self['UPLOADER'] = open('config/UPLOADER').read().strip()

        # conflicts / replaces for zh langpack-split
        if self['PKGCODE'] == 'zh-hans' or self['PKGCODE'] == 'zh-hant':
            if cls != '':
                self['OBSOLETESZHBASE'] = ', language-pack-%s-zh-base (<< 1:9.10)' % self['CLASS']
                self['OBSOLETESZH'] = ', language-pack-%s-zh (<< 1:9.10)' % self['CLASS']
                self['CONFLICTSZH'] = '\nConflicts: language-pack-%s-zh (<< 1:9.10)' % self['CLASS']
            else:
                self['OBSOLETESZHBASE'] = ', language-pack-zh-base (<< 1:9.10)'
                self['OBSOLETESZH'] = ', language-pack-zh (<< 1:9.10)'
                self['CONFLICTSZH'] = '\nConflicts: language-pack-zh (<< 1:9.10)'
            self['OBSOLETESZHSUPPORT'] = '\nConflicts: language-support-zh (<< 1:9.10)\nReplaces: language-support-zh (<< 1:9.10)'
            self['OBSOLETESZHFONTS'] = '\nConflicts: language-support-fonts-zh (<< 1:9.10)\nReplaces: language-support-fonts-zh (<< 1:9.10)'
            self['OBSOLETESZHINPUT'] = '\nConflicts: language-support-input-zh (<< 1:9.10)\nReplaces: language-support-input-zh (<< 1:9.10)'
        else:
            self['OBSOLETESZHBASE'] = ''
            self['OBSOLETESZH'] = ''
            self['CONFLICTSZH'] = ''
            self['OBSOLETESZHSUPPORT'] = ''
            self['OBSOLETESZHFONTS'] = ''
            self['OBSOLETESZHINPUT'] = ''

        # support dependencies
        try:
            #deps = [l.strip() for l in open('support-depends/' + self['PKGNAME'])
            #    if len(l) > 0]
            #self['SUPDEPS'] = ', '.join(deps)
            tags = {}
            f = open('support-depends/' +self['RELEASE'] +'/' + self['PKGNAME'])
            for l in f:
                l = l.strip()
                if len(l) > 0:
                    (tag, pkg) = l.split(':')
                    tags.setdefault(tag, []).append(pkg)
            f.close()

            if Decimal(self['RELEASEVERSION']) < Decimal('9.10'):
                tagmap = {'fn':'FNDEPS',
                          'wa':'WADEPS',
                          'im':'IMDEPS',
                          'tr':'TRDEPS',
                          'ex':'EXDEPS'}

                tag2file = {'fn':'fonts',
                            'wa':'writing',
                            'im':'input',
                            'tr':'translations',
                            'ex':'extra'}
            else:
                tagmap = {'fn':'FNDEPS',
                          'wa':'WADEPS',
                          'im':'IMDEPS'}

                tag2file = {'fn':'fonts',
                            'wa':'writing',
                            'im':'input'}

            supdeps = []
            for key in tagmap.keys():
                if key in tags:
                    self[tagmap[key]] = ', '.join(tags[key])
                    if key == 'ex':
                        self['SUGDEPS'] = ', language-support-extra-%s' % (self['PKGNAME'])
                    else:
                        supdeps += ['language-support-%s-%s' % (tag2file[key], self['PKGNAME'])]
            self['SUPDEPS'] = ', '.join(supdeps)
        except IOError:
            pass

    def __getitem__(self, item):
        # return empty string as default
        return self.macros.get(item, '')

    def __setitem__(self, item, value):
        self.macros[item] = value

    def __contains__(self, item):
        return self.macros.__contains__(item)

    def subst_string(self, s):
        '''Substitute all macros in given string.'''

        re_macro = re.compile('%([A-Z]+)%')
        while 1:
            m = re_macro.search(s)
            if m:
                s = s[:m.start(1)-1] + self[m.group(1)] + s[m.end(1)+1:]
            else:
                break

        return s

    def subst_file(self, file):
        '''Substitute all macros in given file.'''

        s = open(file).read()
        open(file, 'w').write(self.subst_string(s))

    def subst_tree(self, root):
        '''Substitute all macros in given directory tree.'''

        for path, dirs, files in os.walk(root):
            for f in files:
                self.subst_file(os.path.join(root, path, f))

if __name__ == '__main__':
    for locale in ['de', 'de_DE', 'de_DE.UTF-8', 'de_DE.UTF-8@euro',
        'fr_BE@latin', 'zh_CN.UTF-8', 'zh_TW.UTF-8', 'zh_HK.UTF-8', 'zh_hans',
        'zh_hant']:
        for cl in ['', 'gnome', 'kde']:
            l = LangpackMacros(locale, cl, 'karmic', '9.10')
            print '-------', locale, cl, '---------------'
            template = '"language-pack-%PKGNAME%: %CLASSNAMESPACE%packages for %LANG% %CCODE% %PCOUNTRY% %RELEASEVERSION%"'
            print 'string:', l.subst_string(template)

            open('testtest', 'w').write(template)
            l.subst_file('testtest')
            print 'file  :', open('testtest').read()
            os.unlink('testtest')
